
#ifndef __TXP__MIDDLEEND_H
    #define __TXP__MIDDLEEND_H


    #include <string>
    #include <list>
    #include <map>
    #include <stack>
    #include <time.h>
    #include "encoding.h"
    #include "backends/backends.h"


class TXPMiddleEnd;


class AbstractPageContent
{
    friend class TXPMiddleEnd;
    public:
        AbstractPageContent();
        virtual ~AbstractPageContent();
        virtual void WriteToBackend(TXPMiddleEnd* middleend, TXPBackend* backend);
};

class AbstractTextArea : public AbstractPageContent
{
    friend class TXPMiddleEnd;
    protected:
        float left;
        float top;
        std::wstring text;
        TXPFontSettings* font;
    public:
        AbstractTextArea(float left, float top, const std::wstring& text, TXPFontSettings* font);
        ~AbstractTextArea();
        void WriteToBackend(TXPMiddleEnd* middleend, TXPBackend* backend);
};

class AbstractPageLink : public AbstractPageContent
{
    friend class TXPMiddleEnd;
    protected:
        float left;
        float top;
        float width;
        float height;
        std::wstring label;
    public:
        AbstractPageLink(float left, float top, float width, float height, const std::wstring& label);
        ~AbstractPageLink();
        void WriteToBackend(TXPMiddleEnd* middleend, TXPBackend* backend);
};

class AbstractHyperlink : public AbstractPageLink
{
    friend class TXPMiddleEnd;
    public:
        AbstractHyperlink(float left, float top, float width, float height, const std::wstring& url);
        ~AbstractHyperlink();
        void WriteToBackend(TXPMiddleEnd* middleend, TXPBackend* backend);
};

class AbstractPage
{
    friend class TXPMiddleEnd;
    protected:
        std::list<AbstractPageContent*> Contents;

        int PageNumber;
        float PageWidth_cm;
        float PageHeight_cm;
        float PageResolution_dpi;

        float MarginTop_cm;
        float MarginLeft_cm;
        float MarginRight_cm;
        float MarginBottom_cm;

    public:
        AbstractPage(int PageNumber, float PageWidth, float PageHeight, float MarginTop, float MarginLeft, float MarginRight, float MarginBottom, float Resolution_dpi);
        virtual ~AbstractPage();

        void WriteToBackend(TXPMiddleEnd* middleend, TXPBackend* backend);
        void PutText(float left, float top, const std::wstring& text, TXPFontSettings* font);
        void PutPageLink(float left, float top, float width, float height, const std::wstring& label);
        void PutHyperlink(float left, float top, float width, float height, const std::wstring& url);
        void PutBookmark( int depthRelativeToPrevious, const std::wstring& Caption);
        int GetPageNumber();
};

class AbstractBookmark : public AbstractPageContent
{
    friend class TXPMiddleEnd;
    protected:
        AbstractPage* Page;
        std::wstring Caption;
        int depthRelativeToPrevious;
    public:
        AbstractBookmark(AbstractPage* Page, const std::wstring& Caption, int depthRelativeToPrevious);
        ~AbstractBookmark();
        void WriteToBackend(TXPMiddleEnd* middleend, TXPBackend* backend);
};



class TXPMiddleEnd
{
    friend class AbstractBookmark;
    protected:
        int NextPageNumber;
        std::list<AbstractPage*> Pages;
        std::map<std::wstring, AbstractPage*> PageLinks;
        AbstractPage* CurrentPage;
        TXPFontSettings* CurrentFont;

        float left_cm;
        float top_cm;

        float DefaultWidth_cm;
        float DefaultHeight_cm;
        float DefaultResolution_dpi;

        float DefaultMarginTop_cm;
        float DefaultMarginLeft_cm;
        float DefaultMarginRight_cm;
        float DefaultMarginBottom_cm;

        TXPFontSettings* DefaultFont;
        std::stack<int> BookmarkHandleStack;

    public:
        TXPMiddleEnd(float DefaultWidth_cm = 21.59f, float DefaultHeight_cm = 27.94f, float DefaultResolution_dpi=72.0f);
        ~TXPMiddleEnd();

        void AddText( float left, float top, const std::wstring& text, TXPFontSettings* font = NULL);
        void AddText( const std::wstring& text, TXPFontSettings* font = NULL);
        void AddLabel(const std::wstring& label);
        void AddPageLink( float left, float top, float width, float height, const std::wstring& label);
        int GetPageNumber(const std::wstring& label);
        void AddHyperlink( float left, float top, float width, float height, const std::wstring& url);
        void AddBookmark( int depthRelativeToPrevious, const std::wstring& Caption);

        void NewPage();
        void NewPage(float Width, float Height);
        void NewPage(float MarinTop, float MarginLeft, float MarginRight, float MarginBottom);
        void NewPage(float Width, float Height, float MarinTop, float MarginLeft, float MarginRight, float MarginBottom);
        void NewPage(float Width, float Height, float MarinTop, float MarginLeft, float MarginRight, float MarginBottom, float Resolution_dpi);

        void ProduceResult(TXPBackend* backend);
        std::wstring Author;
        std::wstring Title;
        time_t CreationTime;
};


#endif