
#ifndef __TXP__TXP_H
    #define __TXP__TXP_H


    #include <iostream>
    #include <string>
    #include <sstream>
    #include <map>
    #include <math.h>

    #include"../encoding.h"
    #include"../middleend.h"

    namespace yy { class TXPParser; }


    class TXPProcedureDefinition;


    class TXPContext
    {
        public:
            TXPContext(TXPMiddleEnd* middleend);
            ~TXPContext();

            std::map<std::wstring, float> FloatVars;
            std::map<std::wstring, TXPProcedureDefinition*> Procedures;
            TXPMiddleEnd* middleend;
    };


    // =========================================================================================================
    // =========================================================================================================
    // =========================================================================================================


    class TXPElement
    {
        public:
            TXPElement();
            virtual ~TXPElement();
            virtual void Execute(TXPContext* context) = 0;
    };


    // =========================================================================================================
    // =========================================================================================================
    // =========================================================================================================


    class TXPNode : public TXPElement
    {
        friend class yy::TXPParser;
        private:
            TXPNode* prev;
            TXPElement* element;

        public:
            TXPNode(TXPNode* prev, TXPElement* element);
            ~TXPNode();
            void Execute(TXPContext* context);
    };


    // =========================================================================================================
    // =========================================================================================================
    // =========================================================================================================


    template<class T> class TXPCType : public TXPElement
    {
        friend class yy::TXPParser;
        private:
            T* value;
        public:
            TXPCType(T* value);
            ~TXPCType();
            void Execute(TXPContext* context);
    };

    template<class T> TXPCType<T>::TXPCType(T* value)
        : TXPElement()
    {
        this->value = value;
    }

    template<class T> TXPCType<T>::~TXPCType()
    {
        delete value;
    }

    template<class T> void TXPCType<T>::Execute(TXPContext* context)
    {
        if(context != NULL && context->middleend != NULL)
        {
            std::wstringstream ws;
            ws << (*value);
            context->middleend->AddText( ws.str() );
        }
    }

    typedef TXPCType<std::wstring> TXPString;
    typedef TXPCType<float> TXPFloat;
    typedef TXPCType<int> TXPInt;


    // =========================================================================================================
    // =========================================================================================================
    // =========================================================================================================


    class TXPBlock : public TXPElement
    {
        private:
            TXPNode* block;
        public:
            TXPBlock(TXPNode* block);
            ~TXPBlock();
            void Execute(TXPContext* context);
    };


    // =========================================================================================================
    // =========================================================================================================
    // =========================================================================================================


    class TXPStatement : public TXPElement
    {
        public:
            TXPStatement();
            ~TXPStatement();
            void Execute(TXPContext* context) = 0;
    };


    // =========================================================================================================
    // =========================================================================================================
    // =========================================================================================================


    class TXPBool
    {
        public:
            TXPBool();
            virtual ~TXPBool();
            virtual bool Evaluate(TXPContext* context) = 0;
    };

    class TXPBoolTrue : public TXPBool
    {
        public:
            TXPBoolTrue();
            ~TXPBoolTrue();
            bool Evaluate(TXPContext* context);
    };

    class TXPBoolFalse : public TXPBool
    {
        public:
            TXPBoolFalse();
            ~TXPBoolFalse();
            bool Evaluate(TXPContext* context);
    };

    class TXPBoolAnd : public TXPBool
    {
        protected:
            TXPBool* a;
            TXPBool* b;
        public:
            TXPBoolAnd(TXPBool* a, TXPBool* b);
            ~TXPBoolAnd();
            bool Evaluate(TXPContext* context);
    };

    class TXPBoolOr : public TXPBoolAnd
    {
        public:
            TXPBoolOr(TXPBool* a, TXPBool* b);
            ~TXPBoolOr();
            bool Evaluate(TXPContext* context);
    };

    class TXPBoolNot : public TXPBool
    {
        private:
            TXPBool* a;
        public:
            TXPBoolNot(TXPBool* a);
            ~TXPBoolNot();
            bool Evaluate(TXPContext* context);
    };



    // =========================================================================================================
    // =========================================================================================================
    // =========================================================================================================


    class TXPArithmeticExpression : public TXPStatement
    {
        public:
            TXPArithmeticExpression();
            virtual ~TXPArithmeticExpression();
            virtual float Evaluate(TXPContext* context) = 0;
            void Execute(TXPContext* context);
    };

    class TXPArithmeticPlus : public TXPArithmeticExpression
    {
        protected:
            TXPArithmeticExpression* a;
            TXPArithmeticExpression* b;
        public:
            TXPArithmeticPlus(TXPArithmeticExpression* a, TXPArithmeticExpression* b);
            ~TXPArithmeticPlus();
            float Evaluate(TXPContext* context);
    };

    class TXPArithmeticMinus : public TXPArithmeticPlus
    {
        public:
            TXPArithmeticMinus(TXPArithmeticExpression* a, TXPArithmeticExpression* b);
            ~TXPArithmeticMinus();
            float Evaluate(TXPContext* context);
    };

    class TXPArithmeticMult : public TXPArithmeticPlus
    {
        public:
            TXPArithmeticMult(TXPArithmeticExpression* a, TXPArithmeticExpression* b);
            ~TXPArithmeticMult();
            float Evaluate(TXPContext* context);
    };

    class TXPArithmeticDiv : public TXPArithmeticPlus
    {
        public:
            TXPArithmeticDiv(TXPArithmeticExpression* a, TXPArithmeticExpression* b);
            ~TXPArithmeticDiv();
            float Evaluate(TXPContext* context);
    };

    class TXPArithmeticMod : public TXPArithmeticPlus
    {
        public:
            TXPArithmeticMod(TXPArithmeticExpression* a, TXPArithmeticExpression* b);
            ~TXPArithmeticMod();
            float Evaluate(TXPContext* context);
    };

    class TXPArithmeticVariable : public TXPArithmeticExpression
    {
        private:
            std::wstring* name;
        public:
            TXPArithmeticVariable(std::wstring* name);
            ~TXPArithmeticVariable();
            float Evaluate(TXPContext* context);
    };

    class TXPArithmeticConstant : public TXPArithmeticExpression
    {
        private:
            float* value;
        public:
            TXPArithmeticConstant(float* value);
            ~TXPArithmeticConstant();
            float Evaluate(TXPContext* context);
    };


    // =========================================================================================================
    // =========================================================================================================
    // =========================================================================================================


    class TXPRelation : public TXPBool
    {
        protected:
            TXPArithmeticExpression* a;
            TXPArithmeticExpression* b;
            virtual bool Combine(float a, float b) = 0;
        public:
            TXPRelation(TXPArithmeticExpression* a, TXPArithmeticExpression* b);
            ~TXPRelation();
            bool Evaluate(TXPContext* context);
    };

    class TXPRelationEQ : public TXPRelation
    {
        protected:
            bool Combine(float a, float b);
        public:
            TXPRelationEQ(TXPArithmeticExpression* a, TXPArithmeticExpression* b);
            ~TXPRelationEQ();
    };

    class TXPRelationNEQ : public TXPRelation
    {
        protected:
            bool Combine(float a, float b);
        public:
            TXPRelationNEQ(TXPArithmeticExpression* a, TXPArithmeticExpression* b);
            ~TXPRelationNEQ();
    };

    class TXPRelationLT : public TXPRelation
    {
        protected:
            bool Combine(float a, float b);
        public:
            TXPRelationLT(TXPArithmeticExpression* a, TXPArithmeticExpression* b);
            ~TXPRelationLT();
    };

    class TXPRelationLE : public TXPRelation
    {
        protected:
            bool Combine(float a, float b);
        public:
            TXPRelationLE(TXPArithmeticExpression* a, TXPArithmeticExpression* b);
            ~TXPRelationLE();
    };

    class TXPRelationGE : public TXPRelation
    {
        protected:
            bool Combine(float a, float b);
        public:
            TXPRelationGE(TXPArithmeticExpression* a, TXPArithmeticExpression* b);
            ~TXPRelationGE();
    };

    class TXPRelationGT : public TXPRelation
    {
        protected:
            bool Combine(float a, float b);
        public:
            TXPRelationGT(TXPArithmeticExpression* a, TXPArithmeticExpression* b);
            ~TXPRelationGT();
    };


    // =========================================================================================================
    // =========================================================================================================
    // =========================================================================================================


    class TXPProcedureCall : public TXPStatement
    {
        private:
            std::wstring* name;
        public:
            TXPProcedureCall(std::wstring* name);
            virtual ~TXPProcedureCall();
            void Execute(TXPContext* context);
    };


    class TXPProcedureDefinition : public TXPStatement
    {
        friend class TXPProcedureCall;
        private:
            std::wstring* name;
            TXPElement* block;
        public:
            TXPProcedureDefinition(std::wstring* name, TXPElement* block);
            virtual ~TXPProcedureDefinition ();
            void Execute(TXPContext* context);
    };


    class TXPAssignment : public TXPStatement
    {
        private:
            std::wstring* name;
            TXPArithmeticExpression* expr;
        public:
            TXPAssignment(std::wstring* name, TXPArithmeticExpression* expr);
            virtual ~TXPAssignment();
            void Execute(TXPContext* context);
    };


    class TXPFor : public TXPStatement
    {
        private:
            TXPStatement* a;
            TXPBool* cond;
            TXPStatement* b;
            TXPElement* block;
        public:
            TXPFor(TXPStatement* a, TXPBool* cond, TXPStatement* b, TXPElement* block);
            ~TXPFor();
            void Execute(TXPContext* context);
    };


    class TXPIfThenElse : public TXPStatement
    {
        friend class yy::TXPParser;
        private:
            TXPBool* cond;
            TXPElement* thenblock;
            TXPElement* elseblock;
        public:
            TXPIfThenElse(TXPBool* cond, TXPElement* thenblock, TXPElement* elseblock);
            ~TXPIfThenElse();
            void Execute(TXPContext* context);
    };


    class TXPUnicode : public TXPStatement
    {
        private:
            int* codepoint;
        public:
            TXPUnicode(int* codepoint);
            ~TXPUnicode();
            void Execute(TXPContext* context);
    };


    class TXPNewpage : public TXPStatement
    {
        private:
            float* width;
            float* height;
        public:
            TXPNewpage(float* width, float* height);
            ~TXPNewpage();
            void Execute(TXPContext* context);
    };


    class TXPLabel : public TXPStatement
    {
        private:
            std::wstring* name;
        public:
            TXPLabel(std::wstring* name);
            ~TXPLabel();
            void Execute(TXPContext* context);
    };


    class TXPPageLink : public TXPStatement
    {
        protected:
            float* left;
            float* top;
            float* width;
            float* height;
            std::wstring* name;

        public:
            TXPPageLink(float* left, float* top, float* width, float* height, std::wstring* name);
            ~TXPPageLink();
            void Execute(TXPContext* context);
    };


    class TXPHyperlink : public TXPPageLink
    {
        public:
            TXPHyperlink(float* left, float* top, float* width, float* height, std::wstring* url);
            ~TXPHyperlink();
            void Execute(TXPContext* context);
    };

    
    class TXPBookmark : public TXPStatement
    {
        protected:
            int* depthRelativeToPrevious;
            std::wstring* Caption;
        public:
            TXPBookmark(int* depthRelativeToPrevious, std::wstring* Caption);
            ~TXPBookmark();
            void Execute(TXPContext* context);
    };






#endif